#include <windows.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include "usersurf.h"


/*
Written by M. N. AKRAM
Feb, 2018

*/

int __declspec(dllexport) APIENTRY UserDefinedSurface(USER_DATA *UD, FIXED_DATA *FD);

/* a generic Snells law refraction routine */
int Refract(double thisn, double nextn, double *l, double *m, double *n, double ln, double mn, double nn);

BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
	{
   return TRUE;
   }

/* this DLL models a Behrami GRIN Lens ZEMAX surface type with Dispersion */

int __declspec(dllexport) APIENTRY UserDefinedSurface(USER_DATA *UD, FIXED_DATA *FD)
{
	int i;
	double power, a, b, c, d, ta, zc, rho;
	double tp, x, y, z;
	double age, wave_nm, nc, ns, Ac, Bc, Cc, Dc, As, Bs, Cs, Ds;
	double Ra, ka, Rp, kp, Zcmax, Ba, Bp;
	double alpha_A0, alpha_A1, alpha_A2, alpha_A3, alpha_P0, alpha_P1, alpha_P2, alpha_P3, beta_A0, beta_A1, beta_P0, beta_P1;
	double eta_A, eta_A2, eta_P, eta_P2;
	double A1, A2, A3, A4, B1, B2, B3, B4;
	double AA,BB,CC,DD;
	double pi = acos(-1.0);
	double p, Index,ja,jb,jc,rho_ap,eta_ap;
	double dxyz;
	double Indexn;
	double roota, rootp;


	switch (FD->type)
	{
	case 0:
		/* ZEMAX is requesting general information about the surface */
		switch (FD->numb)
		{
		case 0:
			/* ZEMAX wants to know the name of the surface */
			 /* do not exceed 12 characters */
			strcpy(UD->string, "BehrGrin");
			break;
		case 1:
			/* ZEMAX wants to know if this surface is rotationally symmetric */
		   /* it is, so return any character in the string; otherwise, return a null string */
			strcpy(UD->string, "1"); //simetria de revolucion
			break;
		case 2:
			/* ZEMAX wants to know if this surface is a gradient index media */
		   /* it is , so return a 1 string */
			UD->string[0] = '1';	// Con gradiente de indice
			break;
		}
		break;

	case 1:
		/* ZEMAX is requesting the names of the parameter columns */
		 /* the value FD->numb will indicate which value ZEMAX wants. */
		 /* returning a null string indicates that the parameter is unused. */
		switch (FD->numb)
		{
		case 1:
			strcpy(UD->string, "DeltaT");
			break;
		case 2:
			strcpy(UD->string, "p expnt");
			break;
		case 3:
			strcpy(UD->string, "ta");
			break;
		case 4:
			strcpy(UD->string, "Rpost");
			break;
		case 5:
			strcpy(UD->string, "kpost");
			break;
		case 6:
			strcpy(UD->string, "n_surf");
			break;
		case 7:
			strcpy(UD->string, "n_cent");
			break;

		default:
			UD->string[0] = '\0';
			break;
		}
		break;

	case 2:
		/* ZEMAX is requesting the names of the extra data columns */
		 /* the value FD->numb will indicate which value ZEMAX wants. */
		 /* they are all "Unused" for this surface type */
		 /* returning a null string indicates that the extradata value is unused. */
		switch (FD->numb)
		{
		case 1:
			strcpy(UD->string, "Age");
			break;

		case 2:
			strcpy(UD->string, "Acent");
			break;

		case 3:
			strcpy(UD->string, "Bcent");
			break;
		case 4:
			strcpy(UD->string, "Ccent");
			break;
		case 5:
			strcpy(UD->string, "Dcent");
			break;
		case 6:
			strcpy(UD->string, "Asurf");
			break;
		case 7:
			strcpy(UD->string, "Bsurf");
			break;
		case 8:
			strcpy(UD->string, "Csurf");
			break;
		case 9:
			strcpy(UD->string, "Dsurf");
			break;
		case 10:
			strcpy(UD->string, "dxyz");
			break;
		case 11:
			strcpy(UD->string, "Root#Ant");
			break;
		case 12:
			strcpy(UD->string, "Root#Post");
			break;


		default:
			UD->string[0] = '\0';
			break;
		}
		break;

	case 3:
		/* ZEMAX wants to know the sag of the surface */
		 /* if there is an alternate sag, return it as well */
		 /* otherwise, set the alternate sag identical to the sag */
		 /* The sag is sag1, alternate is sag2. */

		UD->sag1 = 0.0;
		UD->sag2 = 0.0;

		/* if a plane, just return */
		if (FD->cv == 0) return(0);
		b = UD->x * UD->x + UD->y * UD->y;
		a = 1 - (1 + FD->k)*FD->cv*FD->cv*b;
		if (a < 0) return(-1);
		UD->sag1 = (FD->cv*b) / (1 + sqrt(a));
		if (a != 1.0) UD->sag2 = (FD->cv*b) / (1 - sqrt(a));
		break;

	case 4:
		/* ZEMAX wants a paraxial ray trace to this surface */
		 /* x, y, z, and the optical path are unaffected, at least for this surface type */
		 /* for paraxial ray tracing, the return z coordinate should always be zero. */
		 /* paraxial surfaces are always planes with the following normals */

		UD->ln = 0.0;
		UD->mn = 0.0;
		UD->nn = -1.0;
		power = (FD->n2 - FD->n1)*FD->cv;
		if ((UD->n) != 0.0)
		{
			(UD->l) = (UD->l) / (UD->n);
			(UD->m) = (UD->m) / (UD->n);

			(UD->l) = (FD->n1*(UD->l) - (UD->x)*power) / (FD->n2);
			(UD->m) = (FD->n1*(UD->m) - (UD->y)*power) / (FD->n2);

			/* normalize */
			(UD->n) = sqrt(1 / (1 + (UD->l)*(UD->l) + (UD->m)*(UD->m)));
			/* de-paraxialize */
			(UD->l) = (UD->l)*(UD->n);
			(UD->m) = (UD->m)*(UD->n);
		}
		break;

	case 5:
		/* ZEMAX wants a real ray trace to this surface */
		if (FD->cv == 0.0)
		{
			UD->ln = 0.0;
			UD->mn = 0.0;
			UD->nn = -1.0;
			if (Refract(FD->n1, FD->n2, &UD->l, &UD->m, &UD->n, UD->ln, UD->mn, UD->nn))
				return(-FD->surf);
			return(0);
		}
		/* okay, not a plane. */
		a = (UD->n) * (UD->n) * FD->k + 1;
		b = ((UD->n) / FD->cv) - (UD->x) * (UD->l) - (UD->y) * (UD->m);
		c = (UD->x) * (UD->x) + (UD->y) * (UD->y);
		d = b * b - a * c;
		if (d < 0)
			return(FD->surf);  /* ray missed this surface */

		if (FD->cv > 0)
			ta = c / (b + sqrt(d));
		else
			ta = c / (b - sqrt(d));

		(UD->x) = (UD->l) * ta + (UD->x);
		(UD->y) = (UD->m) * ta + (UD->y);
		(UD->z) = (UD->n) * ta + (UD->z);
		UD->path = ta;
		zc = (UD->z) * FD->cv;
		d = zc * FD->k * (zc * (FD->k + 1) - 2) + 1;
		c = FD->cv / sqrt(d);
		UD->ln = (UD->x) * c;
		UD->mn = (UD->y) * c;
		UD->nn = ((UD->z) - ((1 / FD->cv) - (UD->z) * FD->k)) * c;

		if (Refract(FD->n1, FD->n2, &UD->l, &UD->m, &UD->n, UD->ln, UD->mn, UD->nn))
			return(-FD->surf);
		break;

	case 6:
		/* ZEMAX wants the index, dn/dx, dn/dy, and dn/dz at the given x, y, z. */

		ta = FD->param[3]; /*Anterior thickness of GRIN*/
		tp = FD->thic - ta; /*Posterior thickness of GRIN*/
		p = FD->param[2];

		x = UD->x; /*coordinates values*/
		y = UD->y;
		z = UD->z - ta; /*shift origin inside lens*/


		wave_nm = FD->wavelength * 1000.0; /*wavelength nm*/

		age = FD->xdata[1]; /*years*/

		Ac = 1.40965 + 3.55E-4*age - 7.5E-6*age*age; /*update dispersion coeficients*/
		Bc = FD->xdata[3];
		Cc = FD->xdata[4];
		Dc = FD->xdata[5];
		As = FD->xdata[6];
		Bs = FD->xdata[7];
		Cs = FD->xdata[8];
		Ds = FD->xdata[9];

		/*center index nc, surface index ns, dispersion dependent */

		nc = Ac + Bc / (wave_nm*wave_nm) + Cc / (wave_nm*wave_nm*wave_nm*wave_nm) + Dc / (wave_nm*wave_nm*wave_nm*wave_nm*wave_nm*wave_nm);
		ns = As + Bs / (wave_nm*wave_nm) + Cs / (wave_nm*wave_nm*wave_nm*wave_nm) + Ds / (wave_nm*wave_nm*wave_nm*wave_nm*wave_nm*wave_nm);


		Ra = 1.0 / FD->cv;
		ka = FD->k;
		Rp = -FD->param[4]; /*switch the sign*/
		kp = FD->param[5];

		roota= FD->xdata[11];
		rootp= FD->xdata[12];


		AA = ((32.0 * Ra*Rp) / 9.0 - (16.0 * Ra*ta) / 9.0 - (16.0 * Ra*tp) / 9.0 -
			(16.0 * Rp*ta) / 9.0 - (16.0 * Rp*tp) / 9.0 + (8.0 * ta*tp) / 9.0 + (4.0 * ta*ta * ka) / 9.0 + (4.0 * ta*ta * kp) / 9.0 + (4.0 * tp*tp * ka) / 9.0 +
			(4.0 * tp*tp * kp) / 9.0 + (16.0 * Ra*Ra) / 9.0 + (16.0 * Rp*Rp) / 9.0 + (4.0 * ta*ta) / 9.0 + (4.0 * tp*tp) / 9.0 -
			(16.0 * Ra*ta*kp) / 9.0 - (16.0 * Rp*ta*ka) / 9.0 - (16.0 * Ra*tp*kp) / 9.0 - (16.0 * Rp*tp*ka) / 9.0 +
			(8.0 * ta*tp*ka) / 9.0 + (8.0 * ta*tp*kp) / 9.0 + (4.0 * ta*ta * ka*kp) / 9.0 + (4.0 * tp*tp * ka*kp) / 9.0 + (8.0 * ta*tp*ka*kp) / 9.0);

		BB = 3.0 * sqrt(AA);

		if (ka == kp)
			Zcmax = 1.0E10;
		else

			Zcmax = -(2.0 * ta - 4.0 * Rp - 4.0 * Ra + 2.0 * tp + 2.0 * ta*ka + 2.0 * tp*kp + BB) / (2.0 * (ka - kp));

		Ba = 2.0 / 3.0 * ((1 + ka)*(ta + Zcmax) - Ra) / ((ta + Zcmax)*(ta + Zcmax));
		Bp = 2.0 / 3.0 * ((1 + kp)*(tp - Zcmax) - Rp) / ((tp - Zcmax)*(tp - Zcmax));

		alpha_A0 = -Ba;
		alpha_A1 = 1.0 + ka - 3.0 * Ba*ta;
		alpha_A2 = -2.0 * Ra + 2.0 * (1.0 + ka)*ta - 3.0 * Ba*ta*ta;
		alpha_A3 = -2.0 * Ra*ta + (1.0 + ka)*ta*ta - Ba * ta*ta*ta;

		beta_A0 = 2.0 * pow(alpha_A2, 3.0) - 9.0 * alpha_A1*alpha_A2*alpha_A3 + 27.0 * alpha_A0* alpha_A3* alpha_A3;
		beta_A1 = alpha_A2 * alpha_A2 - 3.0 * alpha_A1*alpha_A3;

		alpha_P0 = Bp;
		alpha_P1 = 1.0 + kp - 3.0 * Bp*tp;
		alpha_P2 = 2.0 * Rp - 2.0 * (1.0 + kp)*tp + 3.0 * Bp*tp*tp;
		alpha_P3 = -2.0 * Rp*tp + (1.0 + kp)*tp*tp - Bp * tp*tp*tp;

		beta_P0 = 2.0 * pow(alpha_P2, 3.0) - 9.0 * alpha_P1* alpha_P2* alpha_P3 + 27.0 * alpha_P0* alpha_P3* alpha_P3;
		beta_P1 = alpha_P2 * alpha_P2 - 3.0 * alpha_P1*alpha_P3;

		rho = sqrt(x * x + y * y);
		
		if (Zcmax < 0.0)
		{
			if (z <= Zcmax)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Index = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= 0.0) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Index = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > Zcmax) && (z < 0.0)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb - sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho <= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Index = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Index = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			if (Index < 1.0) Index = 1.0;

			UD->index = Index;

			/**************************************************************************/

			/*Define derivative dndx numerically */

			dxyz = FD->xdata[10];

			x = x + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= Zcmax)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= 0.0) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > Zcmax) && (z < 0.0)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb - sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho <= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}

			UD->dndx = (Indexn - Index) / dxyz;

			x = x - dxyz;

			/*Define dndy*/

			y = y + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= Zcmax)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= 0.0) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > Zcmax) && (z < 0.0)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb - sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho <= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			UD->dndy = (Indexn - Index) / dxyz;

			y = y - dxyz;

			/*Define dndz*/

			z = z + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= Zcmax)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= 0.0) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > Zcmax) && (z < 0.0)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb - sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho <= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota*2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp * 2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			UD->dndz = (Indexn - Index) / dxyz;

			z = z - dxyz;
		}

		else /*Zcmax +ve*/
			/**********************************************************************************************************************************/
		{

			if (z <= 0.0)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Index = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= Zcmax) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Index = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > 0) && (z < Zcmax)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb + sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho >= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Index = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Index = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			if (Index < 1.0) Index = 1.0;

			UD->index = Index;

			/**************************************************************************/

			/*Define derivative dndx numerically */

			dxyz = FD->xdata[10];

			x = x + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= 0.0)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= Zcmax) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > 0.0) && (z < Zcmax)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb + sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho >= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}

			UD->dndx = (Indexn - Index) / dxyz;

			x = x - dxyz;

			/*Define dndy*/

			y = y + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= 0.0)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= Zcmax) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > 0) && (z < Zcmax)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb + sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho >= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			UD->dndy = (Indexn - Index) / dxyz;

			y = y - dxyz;

			/*Define dndz*/

			z = z + dxyz;

			rho = sqrt(x * x + y * y);

			if (z <= 0.0)  /*pure Anterior region*/
			{

				CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
				DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

				A1 = sqrt(CC);
				A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
				A3 = sqrt(DD);
				A4 = alpha_A2 * z / (3.0 * alpha_A3);

				eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

				eta_A2 = eta_A * eta_A;

				Indexn = nc + (ns - nc) * pow(eta_A2, p);


			}

			if (z >= Zcmax) /*Pure Posterior region*/

			{

				CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
				DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

				B1 = sqrt(CC);
				B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3.0 * alpha_P3*rho*rho - z * z * beta_P1));
				B3 = sqrt(DD);
				B4 = alpha_P2 * z / (3.0 * alpha_P3);

				eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

				eta_P2 = eta_P * eta_P;

				Indexn = nc + (ns - nc) * pow(eta_P2, p);
			}

			if ((z > 0) && (z < Zcmax)) /* mixed region*/
			{

				ja = 2.0 * Ra - 2.0 * (1.0 + ka)*ta + 3.0 * Ba*ta*ta;
				jb = (6.0 * Ba*ta*z - 2.0 * (1.0 + ka)*z);
				jc = 3.0 * Ba*z*z;

				eta_ap = (-jb + sqrt(jb*jb - 4.0*ja*jc)) / (2.0*ja);

				if (eta_ap == 0.0)
					rho_ap = 0.0;
				else
					rho_ap = sqrt(2.0*Ra*eta_ap*(ta*eta_ap + z) - (1.0 + ka)*(ta*eta_ap + z)*(ta*eta_ap + z) + Ba / eta_ap * (ta*eta_ap + z)*(ta*eta_ap + z)*(ta*eta_ap + z));


				if (rho >= rho_ap) /* anterior region*/
				{
					CC = (4.0 * z*z * beta_A1 - 12.0 * alpha_A3*rho*rho) / (9.0 * alpha_A3*alpha_A3);
					DD = (9.0 * alpha_A3*alpha_A3) / (z*z * beta_A1 - 3.0 * alpha_A3*rho*rho);

					A1 = sqrt(CC);
					A2 = (z*z*z * beta_A0 - 9.0 * alpha_A2*alpha_A3*z*rho*rho) / (6.0 * alpha_A3*(3.0 * alpha_A3*rho*rho - z * z * beta_A1));
					A3 = sqrt(DD);
					A4 = alpha_A2 * z / (3.0 * alpha_A3);

					eta_A = (A1*cos(1.0 / 3.0 * (acos(A2*A3)) - roota * 2.0*pi / 3.0) - A4);

					eta_A2 = eta_A * eta_A;

					Indexn = nc + (ns - nc) * pow(eta_A2, p);

				}
				else /* posterior region*/
				{
					CC = (4.0 * z*z * beta_P1 - 12.0 * alpha_P3*rho*rho) / (9.0 * alpha_P3*alpha_P3);
					DD = (9.0 * alpha_P3*alpha_P3) / (z*z * beta_P1 - 3.0 * alpha_P3*rho*rho);

					B1 = sqrt(CC);
					B2 = (z*z*z * beta_P0 - 9.0 * alpha_P2*alpha_P3*z*rho*rho) / (6.0 * alpha_P3*(3 * alpha_P3*rho*rho - z * z * beta_P1));
					B3 = sqrt(DD);
					B4 = alpha_P2 * z / (3.0 * alpha_P3);

					eta_P = (B1*cos(1.0 / 3.0 * (acos(B2*B3)) - rootp*2.0*pi / 3.0) - B4);

					eta_P2 = eta_P * eta_P;

					Indexn = nc + (ns - nc) *pow(eta_P2, p);

				}

			}


			UD->dndz = (Indexn - Index) / dxyz;

			z = z - dxyz;


		}
			break;

      case 7:
      	/* ZEMAX wants the "safe" data. */
         /* this is used by ZEMAX to set the initial values for all parameters and extra data */
         /* when the user first changes to this surface type. */
         /* this is the only time the DLL should modify the data in the FIXED_DATA FD structure */
         
		 		
		 FD->xdata[1] = 24.0; /*Age*/
		 FD->xdata[2] = 1.4139; /*Acent*/
		 FD->xdata[3] = 6.521218E3; /*Bcent*/
		 FD->xdata[4] = -6.11066E8; /*Ccent*/
		 FD->xdata[5] = 5.908191E13; /*Dcent*/
		 FD->xdata[6] = 1.356086; /*Asurf*/
		 FD->xdata[7] = 6.428455E3; /*Bsurf*/
		 FD->xdata[8] = -6.023738E8; /*Csurf*/
		 FD->xdata[9] = 5.824149E13; /*Dsurf*/
		 FD->xdata[10] = 1.0E-3; /* dxyz*/
		 FD->xdata[11] = 0.0; /* Root # Anterior*/
		 FD->xdata[12] = 0.0; /* Root # Posterior*/

		 for (i = 13; i <= 200; i++) FD->xdata[i] = 0.0;
		 
		 
		 FD->param[1] = 0.05;   /* Delta T */
		 FD->param[2] = 2.886;     /* p, exponente de la expresion de Pierscionek */
		 FD->param[3] = 1.4;   /* ta, posicion en el eje del plano de separacion de las dos partes del cristalino */
		 FD->param[4] = -11.111;  /* R posterior, Radio de la cara posterior del cristalino */
		 FD->param[5] = -3.205;	   /* Q posterior, Conicidad de la cara posterior del Cristalino */
		 FD->param[6] = 1.3729; /* indice de refraccion en la superficie */
		 FD->param[7] = 1.4309; /* indice de refraccion en el centro */
		 

         break;
   }
   return 0;
}

int Refract(double thisn, double nextn, double *l, double *m, double *n, double ln, double mn, double nn)
{
double nr, cosi, cosi2, rad, cosr, gamma;
if (thisn != nextn)
	{
	nr = thisn / nextn;
	cosi = fabs((*l) * ln + (*m) * mn + (*n) * nn);
	cosi2 = cosi * cosi;
	if (cosi2 > 1) cosi2 = 1;
	rad = 1 - ((1 - cosi2) * (nr * nr));
	if (rad < 0) return(-1);
	cosr = sqrt(rad);
	gamma = nr * cosi - cosr;
	(*l) = (nr * (*l)) + (gamma * ln);
	(*m) = (nr * (*m)) + (gamma * mn);
	(*n) = (nr * (*n)) + (gamma * nn);
	}
return 0;
}

